@@ -1,11 +1,10 @@
+import inspect
 import typing
 from contextlib import asynccontextmanager
 
 import starlette.types
-from di import AsyncExecutor, BaseContainer
+from di import AsyncExecutor, BaseContainer, JoinedDependant
 from di.api.dependencies import DependantBase
-from di.api.providers import DependencyProviderType
-from starlette.applications import Starlette
 from starlette.datastructures import State
 from starlette.middleware import Middleware
 from starlette.middleware.errors import ServerErrorMiddleware
@@ -32,19 +31,61 @@ from xpresso.routing.router import Router
 from xpresso.routing.websockets import WebSocketRoute
 from xpresso.security._dependants import Security
 
-ExceptionHandler = typing.Callable[[Request, typing.Type[BaseException]], Response]
+ExceptionHandler = typing.Callable[
+    [Request, Exception], typing.Union[Response, typing.Awaitable[Response]]
+]
+ExceptionHandlers = typing.Mapping[
+    typing.Union[typing.Type[Exception], int], ExceptionHandler
+]
 
 
-class App(Starlette):
+def _include_error_middleware(
+    debug: bool,
+    user_middleware: typing.Iterable[Middleware],
+    exception_handlers: ExceptionHandlers,
+) -> typing.Sequence[Middleware]:
+    # user's exception handlers come last so that they can override
+    # the default exception handlers
+    exception_handlers = {
+        RequestValidationError: validation_exception_handler,
+        HTTPException: http_exception_handler,
+        **exception_handlers,
+    }
+
+    error_handler = None
+    for key, value in exception_handlers.items():
+        if key in (500, Exception):
+            error_handler = value
+        else:
+            exception_handlers[key] = value
+
+    return (
+        Middleware(ServerErrorMiddleware, handler=error_handler, debug=debug),
+        *user_middleware,
+        Middleware(ExceptionMiddleware, handlers=exception_handlers, debug=debug),
+    )
+
+
+def _wrap_lifespan_as_async_generator(
+    lifespan: typing.Callable[..., typing.AsyncContextManager[None]]
+) -> typing.Callable[..., typing.AsyncIterator[None]]:
+    async def gen(
+        *args: typing.Any, **kwargs: typing.Any
+    ) -> typing.AsyncIterator[None]:
+        async with lifespan(*args, **kwargs):
+            yield
+
+    sig = inspect.signature(gen)
+    sig = sig.replace(parameters=list(inspect.signature(lifespan).parameters.values()))
+    setattr(gen, "__signature__", sig)
+
+    return gen
+
+
+class App:
     router: Router
-    middleware_stack: starlette.types.ASGIApp
-    openapi: typing.Optional[openapi_models.OpenAPI] = None
-    _debug: bool
+    openapi: typing.Optional[openapi_models.OpenAPI]
     state: State
-    exception_handlers: typing.Mapping[
-        typing.Union[int, typing.Type[Exception]], ExceptionHandler
-    ]
-    user_middleware: typing.Sequence[Middleware]
     container: BaseContainer
 
     def __init__(
@@ -55,13 +96,11 @@ class App(Starlette):
         dependencies: typing.Optional[typing.List[Dependant]] = None,
         debug: bool = False,
         middleware: typing.Optional[typing.Sequence[Middleware]] = None,
-        exception_handlers: typing.Optional[
-            typing.Dict[
-                typing.Union[int, typing.Type[Exception]],
-                ExceptionHandler,
-            ]
+        exception_handlers: typing.Optional[ExceptionHandlers] = None,
+        lifespan: typing.Optional[
+            typing.Callable[..., typing.AsyncContextManager[None]]
         ] = None,
-        lifespan: typing.Optional[DependencyProviderType[None]] = None,
+        include_in_schema: bool = True,
         openapi_version: str = "3.0.3",
         title: str = "API",
         description: typing.Optional[str] = None,
@@ -70,6 +109,44 @@ class App(Starlette):
         docs_url: typing.Optional[str] = "/docs",
         servers: typing.Optional[typing.Iterable[openapi_models.Server]] = None,
     ) -> None:
+        self.container = container or BaseContainer(
+            scopes=("app", "connection", "endpoint")
+        )
+        register_framework_dependencies(self.container)
+        self._setup_run = False
+
+        @asynccontextmanager
+        async def lifespan_ctx(*args: typing.Any) -> typing.AsyncIterator[None]:
+            lifespans = self._setup()
+            self._setup_run = True
+            original_container = self.container
+            async with self.container.enter_scope("app") as container:
+                self.container = container
+                if lifespan is not None:
+                    dep = Dependant(
+                        _wrap_lifespan_as_async_generator(lifespan), scope="app"
+                    )
+                else:
+                    dep = Dependant(lambda: None, scope="app")
+                solved = self.container.solve(
+                    JoinedDependant(
+                        dep,
+                        siblings=[
+                            Dependant(lifespan, scope="app") for lifespan in lifespans
+                        ],
+                    )
+                )
+                try:
+                    await container.execute_async(solved, executor=AsyncExecutor())
+                    yield
+                finally:
+                    # make this cm reentrant for testing purposes
+                    self.container = original_container
+                    self._setup_run = False
+
+        self._debug = debug
+        self.state = State()
+
         routes = list(routes or [])
         routes.extend(
             self._get_doc_routes(
@@ -77,42 +154,19 @@ class App(Starlette):
                 docs_url=docs_url,
             )
         )
-        self._debug = debug
-        self.state = State()
-        self.exception_handlers = (
-            {} if exception_handlers is None else dict(exception_handlers)
+        middleware = _include_error_middleware(
+            debug=debug,
+            user_middleware=middleware or (),
+            exception_handlers=exception_handlers or {},
+        )
+        self.router = Router(
+            routes,
+            dependencies=dependencies,
+            middleware=middleware,
+            include_in_schema=include_in_schema,
+            lifespan=lifespan_ctx,
         )
 
-        self.container = container or BaseContainer(
-            scopes=("app", "connection", "operation")
-        )
-        register_framework_dependencies(self.container)
-        self._setup_run = False
-
-        @asynccontextmanager
-        async def lifespan_ctx(app: Starlette) -> typing.AsyncGenerator[None, None]:
-            self._setup()
-            self._setup_run = True
-            original_container = self.container
-            async with self.container.enter_scope("app") as container:
-                self.container = container
-                if lifespan is not None:
-                    await container.execute_async(
-                        self.container.solve(Dependant(call=lifespan, scope="app")),
-                        executor=AsyncExecutor(),
-                    )
-                try:
-                    yield
-                finally:
-                    # make this cm reentrant for testing purposes
-                    self.container = original_container
-                    self._setup_run = False
-
-        self.router = Router(routes, lifespan=lifespan_ctx, dependencies=dependencies)
-        self.user_middleware = [] if middleware is None else list(middleware)
-        self.middleware_stack = self.build_middleware_stack()  # type: ignore
-        self.add_exception_handler(RequestValidationError, validation_exception_handler)  # type: ignore
-        self.add_exception_handler(HTTPException, http_exception_handler)  # type: ignore
         self.openapi_version = openapi_version
         self.openapi_info = openapi_models.Info(
             title=title,
@@ -120,28 +174,7 @@ class App(Starlette):
             description=description,
         )
         self.servers = servers
-
-    def build_middleware_stack(self) -> starlette.types.ASGIApp:
-        debug = self.debug
-        error_handler = None
-        exception_handlers = {}
-
-        for key, value in self.exception_handlers.items():
-            if key in (500, Exception):
-                error_handler = value
-            else:
-                exception_handlers[key] = value
-
-        middleware = (
-            Middleware(ServerErrorMiddleware, handler=error_handler, debug=debug),
-            *self.user_middleware,
-            Middleware(ExceptionMiddleware, handlers=exception_handlers, debug=debug),
-        )
-
-        app = self.router
-        for cls, options in reversed(middleware):
-            app = cls(app=app, **options)
-        return app
+        self.openapi = None
 
     async def __call__(
         self,
@@ -149,31 +182,42 @@ class App(Starlette):
         receive: starlette.types.Receive,
         send: starlette.types.Send,
     ) -> None:
-        self._setup()
-        if scope["type"] == "http" or scope["type"] == "websocket":
+        scope["app"] = self
+        scope_type = scope["type"]
+        if scope_type == "http" or scope_type == "websocket":
+            if not self._setup_run:
+                self._setup()
             extensions = scope.get("extensions", None) or {}
             scope["extensions"] = extensions
-            xpresso_scope = extensions.get("xpresso", None)
-            if xpresso_scope is None:
-                async with self.container.enter_scope("connection") as container:
-                    xpresso_asgi_extension: XpressoASGIExtension = {
-                        "container": container,
-                        "response_sent": False,
-                    }
-                    extensions["xpresso"] = xpresso_asgi_extension
-                    await super().__call__(scope, receive, send)
-                    xpresso_asgi_extension["response_sent"] = True
-                    return
-        await super().__call__(scope, receive, send)
-
-    def _setup(self) -> None:
-        if self._setup_run:
+            xpresso_asgi_extension: XpressoASGIExtension = extensions.get("xpresso", None) or {}  # type: ignore[assignment]
+            extensions["xpresso"] = xpresso_asgi_extension
+            async with self.container.enter_scope("connection") as container:
+                xpresso_asgi_extension["response_sent"] = False
+                xpresso_asgi_extension["container"] = container
+                await self.router(scope, receive, send)
+                xpresso_asgi_extension["response_sent"] = True
             return
-        for route in visit_routes([self.router]):
+        else:  # lifespan
+            await self.router(scope, receive, send)
+
+    def _setup(self) -> typing.List[typing.Callable[..., typing.AsyncIterator[None]]]:
+        lifespans: typing.List[typing.Callable[..., typing.AsyncIterator[None]]] = []
+        for route in visit_routes(
+            app_type=App, router=self.router, nodes=[self, self.router], path=""
+        ):
             dependencies: typing.List[DependantBase[typing.Any]] = []
-            for router in route.routers:
-                if isinstance(router, Router):
-                    dependencies.extend(router.dependencies)
+            for node in route.nodes:
+                if isinstance(node, Router):
+                    dependencies.extend(node.dependencies)
+                    if node is not self.router:  # avoid circul lifespan calls
+                        lifespan = typing.cast(
+                            typing.Callable[..., typing.AsyncContextManager[None]],
+                            node.lifespan_context,  # type: ignore  # for Pylance
+                        )
+                        if lifespan is not None:
+                            lifespans.append(
+                                _wrap_lifespan_as_async_generator(lifespan)
+                            )
             if isinstance(route.route, Path):
                 for operation in route.route.operations.values():
                     operation.solve(
@@ -184,7 +228,7 @@ class App(Starlette):
                         ],
                         container=self.container,
                     )
-            if isinstance(route.route, WebSocketRoute):
+            elif isinstance(route.route, WebSocketRoute):
                 route.route.solve(
                     dependencies=[
                         *dependencies,
@@ -192,19 +236,20 @@ class App(Starlette):
                     ],
                     container=self.container,
                 )
+        return lifespans
 
     async def get_openapi(self) -> openapi_models.OpenAPI:
         return genrate_openapi(
+            visitor=visit_routes(app_type=App, router=self.router, nodes=[self, self.router], path=""),  # type: ignore  # for Pylance
             version=self.openapi_version,
             info=self.openapi_info,
             servers=self.servers,
-            router=self.router,
             security_models=await self.gather_security_models(),
         )
 
     async def gather_security_models(self) -> SecurityModels:
         security_dependants: typing.List[Security] = []
-        for route in visit_routes([self.router]):
+        for route in visit_routes(app_type=App, router=self.router, nodes=[self, self.router], path=""):  # type: ignore[misc]
             if isinstance(route.route, Path):
                 for operation in route.route.operations.values():
                     dependant = operation.dependant
@@ -247,7 +292,7 @@ class App(Starlette):
             openapi_url = openapi_url
 
             async def swagger_ui_html(req: Request) -> HTMLResponse:
-                root_path: str = req.scope.get("root_path", "").rstrip("/")
+                root_path: str = req.scope.get("root_path", "").rstrip("/")  # type: ignore  # for Pylance
                 full_openapi_url = root_path + openapi_url  # type: ignore[operator]
                 return get_swagger_ui_html(
                     openapi_url=full_openapi_url,
